;******************************************************************************
;   This file is a basic code template for code generation on the             *
;   PIC16F916. This file contains the basic code building blocks to build     *
;   upon.                                                                     *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on             *
;   features of the assembler.                                                *
;                                                                             *
;   Refer to the respective data sheet for additional                         *
;   information on the instruction set.                                       *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:        RADIOMETER.asm                                          *
;    Date:            August 26, 2012                                         *
;    File Version:     REV 2                                                  *
;                                                                             *
;    Author:       Ron Newton                                                 *
;    Company:                                                                 *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Files Required: P16F916.INC                                              *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Notes:                                                                   *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Revision History:  Final 1/7/2013
;    REVISED    APRIL 6, 2013  
;	 ADDED LINE 377 CALL RESET_REG
;	 TURN OFF INTERRUPS ON RESET_REG
;	 ADDED CLEARING OF MATH REGISTERS       
;	 REVISED OCTOBER 1, 2013                          						      *
;    CMCON0 REGISTERS NOT CONSISTANT   
;	 SPEAKER MUCH LOUDER                                                                   *
;******************************************************************************


	list		p=16f916      ; list directive to define processor
	#include	<p16f916.inc> ; processor specific variable definitions

;------------------------------------------------------------------------------
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;------------------------------------------------------------------------------    

;1	MCLR	NC
;2	RA0		NC	
;3	RA1		COUNT IN
;4	RA2		NC
;5	RA3		NC
;6	RA4		NC
;7	RA5		NC
;8	GND
;9	RA7		SPEAKER
;10	RA6		NC
;11	RC0		DAC DATA
;12	RC1		DAC CLOCK
;13	RC2		DAC CS
;14	RC3		PROGRAMING SWITCH
;15	RC4		RS PIN 39
;16	RC5		CS PIN 38
;17	RC6		R/W PIN 37
;18	RC7		E	PIN 36
;19 GND
;20	V POS
;21	RB0		PIN 35
;22	RB1		PIN 34
;23	RB2		PIN 33
;24	RB3		PIN 32
;25 RB4		PIN 31
;26	RB5		PIN 30
;27	RB6		PIN	29	CLK
;28	RB7		PIN	28	DAT

	
	__CONFIG _FCMEN_ON & _IESO_ON & _CP_OFF & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT

;------------------------------------------------------------------------------
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0,1,2, and 3.  Each Bank contains 
; Special Function Registers, General Purpose Registers, and Access RAM 
;
;------------------------------------------------------------------------------

;       Bit Equates 
;****************************************************************************
DONE    equ     7               ;done measuring flag
EMPTY   equ     5               ;flag if component is connected
V1      equ     5               ;power for C reference ckt
msb_bit equ     7               ;define for bit 7
lsb_bit equ     0               ;define for bit 0
time	equ		3
sign   equ		4

    CBLOCK 0x20 ; Define GPR variable register locations


;****************************************************************************
;       User Registers
;****************************************************************************
;       Bank 0


FLAGS  	            ;FLAGS register
FLAG1				;FLAG1 REGISTER
;FLAG1,0 = MIN
;FLAG1,1 = HOURS
;FLAG1,2 = DAYS
;FLAG1,3 = SPLATCH FLAG
;FLAG1,4 =	SPEAKER ON OFF FLAG
;FLAG1,5 =	ALARM FLAG
;FLAG1,6 =
;FLAG1,7 =
count  	            ;RS232 register
txreg  	            ;RS232 data register
delay  	            ;RS232 delay register
offset  	        ;table position register
msb    	            ;general delay register
lsb    	            ;general delay register
TIMER1
TIMER2
TIMER3
OUT
ROTATE
DAC_MID
DAC_LO
MISC
SECONDS
MINUTES
HOURS
DAYS
COUNT
MIN_REM_HI
MIN_REM_MID
MIN_REM_LO
MIN_DIS_HI
MIN_DIS_MID
MIN_DIS_LO
HOUR_REM_HI
HOUR_REM_MID
HOUR_REM_LO
HOUR_DIS_MID
HOUR_DIS_HI
HOUR_DIS_LO
DAY_REM_HI
DAY_REM_MID
DAY_REM_LO
DAY_DIS_HI
DAY_DIS_MID
DAY_DIS_LO
COUNT_LO
COUNT_MID
COUNT_DIS_LO
COUNT_DIS_MID
ZERO
TIME1
TIME2
CLICK
;		DOG REGISTERS
DATA1
COUNTER0
COUNTER1
ONES
TENS
HUNDREDS
THOUSANDS
DIGIT_1
DIGIT_2
DIGIT_3
DIGIT_4
DIGIT_5



;       Math related registers
ACCaHI  	           ;24-Bit accumulator a
ACCaMID 	
ACCaLO  	
ACCbHI  	           ;24-Bit accumulator b
ACCbMID 	
ACCbLO  	
ACCcHI  	           ;24-Bit accumulator c
ACCcMID 	
ACCcLO  	
ACCdHI  	           ;24-Bit accumulator d
ACCdMID 	
ACCdLO  	
TENK					;10,000
temp   	           		;temporary storage
    ENDC

W_TEMP         EQU    0x7D        ; w register for context saving (ACCESS)
STATUS_TEMP    EQU    0x7E        ; status used for context saving (ACCESS)
PCLATH_TEMP    EQU    0x7F        ; context saving of PC (ACCESS) 
#DEFINE		RW					PORTC,6		;RW OUT
#DEFINE		RS					PORTC,4		;RS OUT
#DEFINE		E					PORTC,7		;E OUT
#DEFINE		DAC_CS				PORTC,2
#DEFINE		CLK					PORTC,1
#DEFINE		DAC_SI				PORTC,0
#DEFINE		PROGRAM_SWITCH		PORTC,3
#DEFINE		SPEAKER				PORTA,5
;------------------------------------------------------------------------------
; EEPROM INITIALIZATION
;
; The 16F916 has 256 bytes of non-volatile EEPROM, STARTing at address 0x2100
; 
;------------------------------------------------------------------------------

DATAEE    ORG  0x2100
;		DE	0x00, 0x01, 0x02, 0x03


;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

    ORG     0x0000            ; processor reset vector
    PAGESEL init
        goto    init

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------


	org 4



    ; Interrupt context saving
	BANKSEL	 PORTA			  ;change banks
    MOVWF    W_TEMP           ; save off current W register contents
    MOVF     STATUS,W         ; move status register into W register
    MOVWF    STATUS_TEMP      ; save off contents of STATUS register
    MOVF     PCLATH,W         ; move pclath register into w register
    MOVWF    PCLATH_TEMP      ; save off contents of PCLATH register

;------------------------------------------------------------------------------
		BTFSS		PIR2,C2IF
		GOTO		INT0

C0		BSF			FLAG1,3			;SET SPLATCH FLAG
		CLRF		PIR2			;RESET INTERRUPT	
		BANKSEL		CMCON0
		MOVF		CMCON0,1		;READ CMCON0 TO RESET for read
		BANKSEL		PORTA
		CLRF		PIR2			;RESET INTERRUPT		

		GOTO		INT2			;JUMP TO INT2

INT0
		DECFSZ	  	TIMER1,1
		GOTO		INT1	
		MOVLW	    .246
		MOVWF		TIMER1
		DECF		ZERO,1			;TIMER FOR PROGRAM SWITCH
		DECFSZ		SECONDS,1
		GOTO		INT1
		BSF			FLAG1,0			;SET FLAG1,0
		MOVLW		.60
		MOVWF		SECONDS
		DECFSZ		MINUTES,1
		GOTO		INT1
		BSF			FLAG1,1			;SET FLAG1,1
		MOVLW		.60
		MOVWF		MINUTES
		DECFSZ		HOURS,1
		GOTO		INT1
		BSF			FLAG1,2			;SET FLAG1,2
		MOVLW		.24
		MOVWF		HOURS
		
	
INT1
		MOVLW	.3				;CORRECTION FOR TIME
		MOVWF	TMR0
        BCF   INTCON,T0IF        ;Service Timer 0 overflow

;------------------------------------------------------------------------------
INT2
    ; Interrupt context restoring
    MOVF    PCLATH_TEMP,W     ; retrieve copy of PCLATH register
    MOVWF   PCLATH            ; restore pre-isr PCLATH register contents
    MOVF    STATUS_TEMP,W     ; retrieve copy of STATUS register
    MOVWF   STATUS            ; restore pre-isr STATUS register contents
    SWAPF   W_TEMP,F
    SWAPF   W_TEMP,W          ; restore pre-isr W register contents
    RETFIE                    ; return from interrupt
    
;------------------------------------------------------------------------------
; MAIN PROGRAM
;------------------------------------------------------------------------------

init
        Banksel	PORTA			;change banks
        clrf    PORTA           ;clear PORTA, PORTB and PORTC
        clrf    PORTB
		clrf	PORTC
		CLRF	INTCON
		BANKSEL	CMCON0			;change banks
        movlw   0x07            ;turn off comparators, mode 111
        movwf   CMCON0
        BANKSEL	PORTA
        call    delay20         ;wait for comparators to settle

		BANKSEL	OPTION_REG		;change banks
        movlw  B'00010011'    ;TMR0 prescalar 1:16
        movwf   OPTION_REG
		BANKSEL	TRISA			;change banks
        MOVLW	B'00000010'
        MOVWF	TRISA
        clrf    TRISB			;PORTB all outputs
        movlw	b'00001000'
		movwf	TRISC			;PORTC output except RC4
		BANKSEL	VRCON			;change banks
   		MOVLW	B'10101100'		;2.5 VOLTS           ;setup Voltage Reference
        movwf   VRCON
        BANKSEL	LCDCON
        CLRF	LCDCON
        BANKSEL	ANSEL
        MOVLW	B'00000010'
        MOVWF	ANSEL
        BANKSEL	PORTA			;change banks
		movlw	b'11100000'
		movwf	INTCON
        bsf		DAC_CS			;TURN OFF DAC
		CLRF	FLAG1			;CLEAR ALL FLAGS
        BSF		FLAG1,4			;SET SPEAKER FLAG FOR ON
		CLRF	FLAGS
		BSF		SPEAKER			;TURN OFF SPEAKER
		CALL	AnalogOn
		CALL	RESET_REG
		CALL	DOG				;INTIALIZE LCD


;********************TEST AREA*********************
TEST

;**************************************************
		BTFSC	PROGRAM_SWITCH	
		GOTO	START
		BCF		FLAG1,4			;SPEAKER FLAG TURN OFF
		CALL	AnalogOn		;SET ANALOG FOR NO SOUND
		CALL	DOG
		CALL	SPK				;DISPLAY SPEAKER SITUATION
		BTFSS	PROGRAM_SWITCH		
		GOTO	$-1

START
 	    BTFSC	PROGRAM_SWITCH	
		GOTO	S1				;NOT PUSHED JUMP TO S1
		MOVLW	.3				;PUSHED LOAD TIME REG
		MOVWF	ZERO
S0A		BTFSC	PROGRAM_SWITCH	;CHECK TO SEE IF RELEASED
		GOTO	S0				;RELEASE JUMP TO S0
		MOVLW	0				;CHECK FOR BEING HELD FOR 5 SECONDS
		SUBWF	ZERO,0
		BTFSS	3,2
		GOTO	S0A				;NOT RELEASED
		CALL	RESET_REG		;5 SECONDS ZERO ALL
		BTFSS	PROGRAM_SWITCH	;WAIT UNTIL RELEASED
		GOTO	$-1
		GOTO	START
		
S0		INCF	FLAGS,1
		BTFSS	PROGRAM_SWITCH			;WAIT FOR BUTTON TO BE RELEASED
		GOTO	$-1
		CALL	RESET_REG
S1		CALL	MODE	
		BTFSS	FLAG1,0			;CHECK FOR MINUTE FLAG
		GOTO	START			;NOT ZERO REPEAT
		BCF		FLAG1,0			;RESET MINUTE FLAG
		BCF		FLAG1,5			;RESET ALARM FLAG
		MOVF	COUNT_LO,0		;LOADING DAC REGISTERS
		MOVWF	COUNT_DIS_LO
		MOVWF	DAC_LO
		MOVWF	ACCbLO
		MOVF	COUNT_MID,0
		MOVWF	COUNT_DIS_MID
		MOVWF	DAC_MID
		MOVWF	ACCbMID
		CALL	DAC				;WAS ZERO WRITE TO DAC
		CLRF	COUNT_LO
		CLRF	COUNT_MID
;*****************MULTIPLYING TO GET CORRECT POSITION ON READ OUT**********
AA		CLRF	ACCaHI
		CLRF	ACCaMID
		movlw	.100			;MOVE DECIMAL POINT TO LEFT BY 2
		movwf	ACCaLO
		call	Mpy24			;MULTIPLY BY 100 
        
		MOVLW	.105			;FACTOR FOR MICRO-REMS FOR LND 712 TUBE
		MOVWF	 ACCaLO
		CLRF	ACCaMID
		CLRF	ACCaHI
		CALL	Div24			;CALCULATING MICRO-REMS PER MINUTE

;*****************************ADDING TO HOUR REGISTER 
		MOVF	ACCcLO,0
		MOVWF	MIN_REM_LO			;LOAD MINUTE REGISTER
		MOVWF	MIN_DIS_LO
		ADDWF	HOUR_REM_LO,1		;ADD TO HOUR RESISTERS
		BTFSC	3,0
		INCF	HOUR_REM_MID,1		;ADD TO HOUR RESISTERS
		MOVF	ACCcMID,0
		MOVWF	MIN_DIS_MID
		MOVWF	MIN_REM_MID			;LOAD MINUTE REGISTER
		ADDWF	HOUR_REM_MID,1		;ADD TO HOUR RESISTERS
		BTFSC	3,0
		INCF	HOUR_REM_HI,1		;ADD TO HOUR RESISTERS
		MOVF	ACCcHI,0
		MOVWF	MIN_DIS_HI
		MOVWF	MIN_REM_HI			;LOAD MINUTE REGISTER
		ADDWF	HOUR_REM_HI,1		;ADD TO HOUR RESISTERS

		BTFSS	FLAG1,1				;CHECK 60 MINUTE ROLL OVER
		GOTO	START
		BCF		FLAG1,1				;RESET FLAG
		MOVF	HOUR_REM_HI,0		;PUT INTO HOLDING REGISTERS FOR DISPLAY
		MOVWF	HOUR_DIS_HI
		MOVF	HOUR_REM_MID,0
		MOVWF	HOUR_DIS_MID
		MOVF	HOUR_REM_LO,0
		MOVWF	HOUR_DIS_LO
		CLRF	HOUR_REM_HI			;CLEAR HOUR REGISTERS FOR NEXT HOUR
		CLRF	HOUR_REM_MID
		CLRF	HOUR_REM_LO

;*********ADDING TO DAY REGISTERS
		MOVF	HOUR_DIS_LO,0
		ADDWF	DAY_REM_LO,1
		BTFSC	3,0
		INCF	DAY_REM_MID,1
		MOVF	HOUR_DIS_MID,0
		ADDWF	DAY_REM_MID,1
		BTFSC	3,0
		INCF	DAY_REM_HI,1
		MOVF	HOUR_DIS_HI,0
		ADDWF	DAY_REM_HI,1
 		BTFSS	FLAG1,2
		GOTO	START
		BCF		FLAG1,2				;CLEAR FLAG
		MOVF	DAY_REM_HI,0		;PUT INTO HOLDING REGISTERS FOR DISPLAY
		MOVWF	DAY_DIS_HI
		MOVF	DAY_REM_MID,0
		MOVWF	DAY_DIS_MID
		MOVF	DAY_REM_LO,0
		MOVWF	DAY_DIS_LO
		CLRF	DAY_REM_HI			;CLEAR DAY REGISTERS FOR NEXT DAY
		CLRF	DAY_REM_MID
		CLRF	DAY_REM_LO
		GOTO	START


;***************************DETERMINING WHAT MODE IS DISPLAYED
MODE	MOVLW	0
		SUBWF	FLAGS,0
		BTFSC	3,2
		GOTO	CHANGE0			;DISPLAY CPM
		MOVLW	1
		SUBWF	FLAGS,0
		BTFSC	3,2
		GOTO	CHANGE1			;DISPLAY MINUTE MICRO-REMS
		MOVLW	2
		SUBWF	FLAGS,0
		BTFSC	3,2
		GOTO	CHANGE2			;DISPLAY HOUR MICRO-REMS
		MOVLW	3
		SUBWF	FLAGS,0
		BTFSC	3,2
		GOTO	CHANGE3			;DISPLAY DAY MICRO-REMS
		MOVLW	4
		SUBWF	FLAGS,0
		BTFSC	3,2
		CLRF	FLAGS			;RESET FLAGS
		GOTO	MODE

;**************************COUNTS PER MINUTE*******************************
CHANGE0 
		CALL	DOG	
		MOVF	COUNT_DIS_LO,0
		MOVWF	ACCcLO
		MOVF	COUNT_DIS_MID,0
		MOVWF	ACCcMID
        CALL 	DECIMAL			;convert
		CALL	DISPLAYING
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0	
		CALL	SEND
		MOVLW	02CH			;,
		CALL	SEND
		MOVF	DIGIT_3,0	
		CALL	SEND
		MOVF	DIGIT_2,0	
		CALL	SEND
		MOVF	DIGIT_1,0	
		CALL	SEND
		MOVLW	0A0H			;SPACE
		CALL	SEND
		MOVLW	043H			;C
		CALL 	SEND
		MOVLW	050H			;P
		CALL 	SEND
		MOVLW	04DH			;M
		CALL	SEND
	
		CALL	SPLATCH
		MOVF	SECONDS,0
		MOVWF	ACCcLO
		CLRF	ACCcMID
		CALL	DECIMAL
		CALL	DISPLAYING 
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	045H			;E
		CALL	SEND
		MOVLW	043H			;C
		CALL	SEND
		MOVLW	0A0H			;SPACE
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND			
		CALL	SPK
		RETURN
;**************************REM PER MINUTE*****************************	
CHANGE1	
		CALL	DOG	
		MOVF	MIN_DIS_LO,0
		MOVWF	ACCcLO
		MOVF	MIN_DIS_MID,0
		MOVWF	ACCcMID
		MOVF	MIN_DIS_HI,0
		MOVWF	ACCcHI
		CALL	DECIMAL
		CALL	DISPLAYING
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0	
		CALL	SEND
		MOVF	DIGIT_3,0	
		CALL	SEND
		MOVLW	02EH			;.
		CALL	SEND
		MOVF	DIGIT_2,0	
		CALL	SEND
		MOVF	DIGIT_1,0	
		CALL	SEND
		CALL	MICRO
		MOVLW	04DH			;M
		CALL	SEND
		MOVLW	069H			;i
		CALL	SEND
		MOVLW	06EH			;n
		CALL	SEND
		CALL	SPLATCH

		BTFSC	FLAG1,5			;CHECKING ALARM FLAG
		GOTO	CH1				;SET JUMP
		MOVLW	0				;CHECKING IF ABOVE .5 REMS/MIN
		SUBWF	MIN_DIS_HI,0
		BTFSS	3,2
		CALL	ALARM
		MOVLW	.50
		SUBWF	MIN_DIS_LO,0
		BTFSC	3,0
		CALL 	ALARM		
CH1		MOVF	SECONDS,0
		MOVWF	ACCcLO
		CLRF	ACCcMID
		CALL	DECIMAL
		CALL	DISPLAYING 
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	045H			;E
		CALL	SEND
		MOVLW	043H			;C
		CALL	SEND
		MOVLW	0A0H			;SPACE
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND			
		CALL	SPK

		RETURN


;*****************************REM PER HOUR**********************
CHANGE2	
		CALL	DOG	
		MOVLW	0					;CHECK FOR COMPLETE COUNT
		SUBWF	HOUR_DIS_LO,0
		BTFSC	3,2
		GOTO	COUNTING1
		MOVLW	001H ^0FFH			;CHECK IF OVER 640.00
		SUBWF	HOUR_DIS_HI,0
		BTFSS	3,0
		GOTO	CHANGE2A		
		MOVF	HOUR_DIS_LO,0
		MOVWF	ACCcLO
		MOVF	HOUR_DIS_MID,0
		MOVWF	ACCcMID
		MOVF	HOUR_DIS_HI,0
		MOVWF	ACCcHI
		MOVLW	0E8H
		MOVWF	ACCaLO
		MOVLW	03H
		MOVWF	ACCaMID
		CLRF	ACCaHI
		CALL	Div24				;DIVIDING BY 1000 MAKING MILLI-REM
		CALL	DECIMAL
		CALL	DISPLAYING
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0	
		CALL	SEND
		MOVF	DIGIT_3,0	
		CALL	SEND
		MOVLW	02EH			;.
		CALL	SEND
		MOVF	DIGIT_2,0	
		CALL	SEND
		MOVF	DIGIT_1,0	
		CALL	SEND
		CALL	MILI
		MOVLW	048H			;H
		CALL	SEND
		MOVLW	072H			;r
		CALL	SEND		
		CALL	SPLATCH
		CALL	SPK
		RETURN		


CHANGE2A
		MOVF	HOUR_REM_LO,0
		MOVLW	ACCcLO
		MOVF	HOUR_REM_MID,0
		MOVWF	ACCcMID
		MOVF	HOUR_REM_HI,0
		MOVWF	ACCcHI
		CALL	DECIMAL
		CALL	DISPLAYING
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0	
		CALL	SEND
		MOVF	DIGIT_3,0	
		CALL	SEND
		MOVLW	02EH			;.
		CALL	SEND
		MOVF	DIGIT_2,0	
		CALL	SEND
		MOVF	DIGIT_1,0	
		CALL	SEND
		CALL	MICRO
		MOVLW	048H			;H
		CALL	SEND
		MOVLW	072H			;r
		CALL	SEND		
		CALL	SPLATCH
		CALL	SPK
		RETURN	

;*******************************REM PER DAY*********************
CHANGE3
		CALL	DOG	
		MOVLW	0					;CHECK FOR COMPLETE COUNT
		SUBWF	DAY_DIS_LO,0
		BTFSC	3,2
		GOTO	COUNTING2
		MOVLW	001H ^0FFH			;CHECK IF OVER 640.00
		SUBWF	DAY_DIS_HI,0
		BTFSS	3,0
		GOTO	CHANGE3A		
		MOVF	DAY_DIS_LO,0
		MOVWF	ACCcLO
		MOVF	DAY_DIS_MID,0
		MOVWF	ACCcMID
		MOVF	DAY_DIS_HI,0
		MOVWF	ACCcHI
		MOVLW	0E8H
		MOVWF	ACCaLO
		MOVLW	03H
		MOVWF	ACCaMID
		CLRF	ACCaHI
		CALL	Div24				;DIVIDING BY 1000
		CALL	DECIMAL
		CALL	DISPLAYING
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0	
		CALL	SEND
		MOVF	DIGIT_3,0	
		CALL	SEND
		MOVLW	02EH			;.
		CALL	SEND
		MOVF	DIGIT_2,0	
		CALL	SEND
		MOVF	DIGIT_1,0	
		CALL	SEND

		CALL	MILI
		MOVLW	044H			;D
		CALL	SEND
		MOVLW	061H			;a
		CALL	SEND
		MOVLW	079H			;y
		CALL	SEND		
		CALL	SPLATCH
		CALL	SPK
		RETURN	
CHANGE3A
		MOVF	DAY_DIS_LO,0
		MOVWF	ACCcLO
		MOVF	DAY_DIS_MID,0
		MOVWF	ACCcMID
		MOVF	DAY_DIS_HI,0
		MOVWF	ACCcHI
		CALL	DECIMAL
		CALL	DISPLAYING
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0	
		CALL	SEND
		MOVF	DIGIT_3,0	
		CALL	SEND
		MOVLW	02EH			;.
		CALL	SEND
		MOVF	DIGIT_2,0	
		CALL	SEND
		MOVF	DIGIT_1,0	
		CALL	SEND
		CALL	MICRO
		MOVLW	044H			;D
		CALL	SEND
		MOVLW	061H			;a
		CALL	SEND
		MOVLW	079H			;y
		CALL	SEND		
		CALL	SPLATCH
		CALL	SPK
		RETURN	

MICRO	MOVLW	0A0H
		CALL	SEND
		MOVLW	075H			;u
		CALL	SEND
		MOVLW	052H			;R
		CALL	SEND
		MOVLW	02FH			;/
		CALL	SEND
		RETURN


MILI	MOVLW	0A0H
		CALL	SEND
 		MOVLW	06dH			;m
		CALL	SEND
		MOVLW	052H			;R
		CALL	SEND
		MOVLW	02FH			;/
		CALL	SEND
		RETURN

SPK	


		MOVLW		B'10110001'	;SHIFT DRAM
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS		
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	070H			;p
		CALL	SEND
		MOVLW	06BH			;k
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		BTFSC	FLAG1,4			;CHECK FOR SPEAKER BEING OFF
		GOTO	SPK1
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	066H			;f
		CALL	SEND
		MOVLW	066H			;f
		CALL	SEND
		RETURN
SPK1	MOVLW	04FH			;O
		CALL	SEND
		MOVLW	06EH			;n
		CALL	SEND
		RETURN
;***********************DISPLAY COUNTING*********************
COUNTING1
		MOVLW	043H			;C
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	055H			;U
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	049H			;I
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	047H			;G
		CALL	SEND
		CALL	SPLATCH
		MOVF	MINUTES,0
		MOVWF	ACCcLO
		CLRF	ACCcMID
		CALL	DECIMAL
		CALL	DISPLAYING 
		MOVLW	04DH			;M
		CALL	SEND
		MOVLW	069H			;I
		CALL	SEND
		MOVLW	06EH			;N
		CALL	SEND
		MOVLW	0A0H			;SPACE
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND			
		CALL	SPK		
		RETURN		

COUNTING2
		MOVLW	043H			;C
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	055H			;U
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	049H			;I
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	047H			;G
		CALL	SEND
		CALL	SPLATCH
		MOVF	HOURS,0
		MOVWF	ACCcLO
		CLRF	ACCcMID
		CALL	DECIMAL
		CALL	DISPLAYING 
		MOVLW	048H			;H
		CALL	SEND
		MOVLW	072H			;R
		CALL	SEND
		MOVLW	073H			;S
		CALL	SEND
		MOVLW	0A0H			;SPACE
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND			
		CALL	SPK		
		RETURN		
SPLATCH
		MOVLW		B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		BTFSC		FLAG1,3			;CHECK SPLATCH FLAG
		GOTO		SPL1
		MOVLW		0A0H			;BLANK
		CALL		SEND
		MOVLW		0A0H			;BLANK
		CALL		SEND
		BCF			FLAG1,3			;RESET SPLATCH FLAG
		RETURN
SPL1	MOVLW		011H			;MAKE SPLATCH
		CALL		SEND
		MOVLW		0A0H			;BLANK
		CALL		SEND
		BCF			FLAG1,3			;RESET SPLATCH FLAG
		INCFSZ		COUNT_LO,1
		RETURN
		INCF		COUNT_MID,1
		RETURN 

ALARM
		BTFSS	FLAG1,4			;SPEAKER FLAG
		GOTO	ALARM1
		BSF		FLAG1,5			;SET ALARM FLAG
        BANKSEL	CMCON0			;change banks
        movlw   B'00001101'     ;turn comparator on, use voltage ref
        movwf   CMCON0          ;1 inputs				
		movf	CMCON0,F		;for detection of change of comparator
		BANKSEL	PORTA
		bcf		SPEAKER
		MOVLW	.5				;ALARM FOR 5 SECONDS
		MOVWF	ZERO
		MOVLW	0
		SUBWF	ZERO,0
		BTFSS	3,2
		GOTO	$-3
		BSF		SPEAKER
ALARM1	NOP			;NEEDED FOR BANKSEL
	    BANKSEL	CMCON0						;change banks
		movf	CMCON0,F		;for detection of change of comparator TURN ON CLICK
        movlw   B'00001101'     ;turn comparator on, use voltage ref
        movwf   CMCON0          ;1 inputs
		BANKSEL	PORTA
		RETURN

;*****************************END OF MAIN PROGRAM**************
;**************************************************************
AnalogOn
	BTFSC	FLAG1,4
	GOTO	ANA1	
        BANKSEL	CMCON0			;change banks
        movlw  	B'00001101'     ;turn comparator on, use voltage ref
        movwf   CMCON0          ;1 inputs
   		movf	CMCON0,F		;for detection of change of comparator
        BSF		SPEAKER
		GOTO	ANA2
ANA1 	NOP
	    BANKSEL	CMCON0			;change banks
        movlw  	B'00001101'     ;turn comparator on, use voltage ref
        movwf   CMCON0   
		movf	CMCON0,F		;for detection of change of comparator

ANA2	NOP
		BANKSEL	VRCON			;change banks
		MOVLW	B'10101100'		;2.5 VOLTS
        MOVWF    VRCON			;LOAD VRCON
        BANKSEL	PORTA			;change banks
        call    delay20         ;20msec delay
		BANKSEL	CMCON0			;change banks
        movf    CMCON0,F        ;clear comparator mismatch condition
 		BANKSEL	PORTA
        CLRF    PIR2      		;clear comparator interrupt flag
        BANKSEL	PIE2 
        BSF		PIE2,C2IE		;TURN ON INTERRUPT
        BANKSEL	PORTA			;change banks
		movlw	b'11100000'
		movwf	INTCON
        clrf    TMR0            ;clear TMR0 counter
        return



;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Delay Routines                                                     -*
;*--------------------------------------------------------------------------*
;****************************************************************************
vlong   movlw   0xff            ;very long delay, approx 200msec
        movwf   msb
        goto    d1
delay20                         ;20 msec delay
        movlw   .26
        movwf   msb
        goto    d1
delay16                         ;16 msec delay
        movlw   .21
        movwf   msb
        goto    d1
delay1                          ;approx 750nsec delay
        movlw   .1      
        movwf   msb
d1      movlw   0xff
        movwf   lsb
d2      decfsz  lsb,F
        goto    d2
        decfsz  msb,F
        goto    d1
        return
;____________________________________________________________________________

;****************************************************************************


;************************DAC PROGRAM**********************

DAC							;LABEL

	BCF		DAC_CS			;SET CS LOW TO TURN ON
D1	BCF		DAC_SI			;
	CALL	CLOCK_OUT		;WRITE TO DAC
	CALL	CLOCK_OUT		;UNBUFFERED
	BSF		DAC_SI
	CALL	CLOCK_OUT		;GAIN1
	CALL	CLOCK_OUT		;ACTIVE

	MOVLW	4				;FOUR BITS
	MOVWF	ROTATE			;COUNTER
	MOVF	DAC_MID,0		;MOVING DAC MID INTO OUT
	MOVWF	OUT				;MOVE
	BCF		3,0				;CLR C
	RLF		OUT,1
	RLF		OUT,1
	RLF		OUT,1
	RLF		OUT,1
D2	RLF		OUT,1			;ROTATE MSD OUT INTO C
	BTFSS	3,0				;CHECK TO SEE IF C IS HIGH
	BCF		DAC_SI			;ANALOG OUT LOW
	BTFSC	3,0				;CHECK TO SEE IF C IS LOW
	BSF		DAC_SI			;ANALOG OUT LOW
	CALL	CLOCK_OUT		;CLOCK OUT
	DECFSZ	ROTATE,1		;DO 4 TIMES
	GOTO	D2				;DO AGAIN

	MOVLW	8				;EIGHT BITS
	MOVWF	ROTATE			;COUNTER
	MOVF	DAC_LO,0		;MOVING DAC LO INTO OUT
	MOVWF	OUT				;MOVE
	BCF		3,0				;CLR C
D3	RLF		OUT,1			;ROTATE MSD OUT INTO C
	BTFSS	3,0				;CHECK TO SEE IF C IS HIGH
	BCF		DAC_SI			;ANALOG OUT LOW
	BTFSC	3,0				;CHECK TO SEE IF C IS LOW
	BSF		DAC_SI			;ANALOG OUT LOW
	CALL	CLOCK_OUT		;CLOCK OUT
	DECFSZ	ROTATE,1		;DO 8 TIMES
	GOTO	D3				;DO AGAIN
	BSF		DAC_CS			;SET CS HIGH TO TURN LOAD
	RETURN

CLOCK_OUT					;LABEL
	BSF		CLK				;CLOCK HIGH
	BCF		CLK				;CLOCK LOW
	RETURN					;RETURN

	
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     24-bit Addition                                                    -*
;*-                                                                        -*
;*-     Uses ACCa and ACCb                                                 -*
;*-                                                                        -*
;*-     ACCa + ACCb -> ACCb                                                -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Add24
        movf    ACCaLO,W
        addwf   ACCbLO,1          ;add low bytes
        btfsc   STATUS,C        ;add in carry if necessary
        goto    A2
A1      movf    ACCaMID,W
        addwf   ACCbMID,1         ;add mid bytes
        btfsc   STATUS,C        ;add in carry if necessary
        incf    ACCbHI,1
        movf    ACCaHI,W
        addwf   ACCbHI,1          ;add high bytes
        retlw   0
A2      incf    ACCbMID,1
        btfsc   STATUS,Z
        incf    ACCbHI,1
        goto    A1
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*       
;*-     Subtraction ( 24 - 24 -> 24 )                                      -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCd                                              -*
;*-                                                                        -*
;*-     ACCa -> ACCd,                                                      -*
;*-     2's complement ACCa,                                               -*
;*-     call Add24 ( ACCa + ACCb -> ACCb ),                                -*
;*-     ACCd -> ACCa                                                       -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Sub24
        movf    ACCaHI,W                ;Transfer ACCa to ACCd
        movwf   ACCdHI
        movf    ACCaMID,W
        movwf   ACCdMID
        movf    ACCaLO,W
        movwf   ACCdLO
        call    compA                   ;2's complement ACCa
        call    Add24                   ;Add ACCa to ACCb
        movf    ACCdHI,W                ;Transfer ACCd to ACCa
        movwf   ACCaHI
        movf    ACCdMID,W
        movwf   ACCaMID
        movf    ACCdLO,W
        movwf   ACCaLO
        retlw   0
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Multiply ( 24 X 24 -> 56 )                                         -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCc, ACCd                                        -*
;*-                                                                        -*
;*-     ACCa * ACCb -> ACCb,ACCc  56-bit output                            -*
;*-     with ACCb (ACCbHI,ACCbMID,ACCbLO) with 24 msb's and                -*
;*-     ACCc (ACCcHI,ACCcMID,ACCcLO) with 24 lsb's                         -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Mpy24
        call    Msetup
mloop   rrf     ACCdHI,1          ;rotate d right
        rrf     ACCdMID,1
        rrf     ACCdLO,1
        btfsc   STATUS,C        ;need to add?
        call    Add24
        rrf     ACCbHI,1
        rrf     ACCbMID,1
        rrf     ACCbLO,1
        rrf     ACCcHI,1
        rrf     ACCcMID,1
        rrf     ACCcLO,1
        decfsz  temp,1            ;loop until all bits checked
        goto    mloop
        retlw   0

Msetup
        movlw   0x18            ;for 24 bit shifts
        movwf   temp
        movf    ACCbHI,W        ;move ACCb to ACCd
        movwf   ACCdHI
        movf    ACCbMID,W
        movwf   ACCdMID
        movf    ACCbLO,W
        movwf   ACCdLO
        clrf    ACCbHI
        clrf    ACCbMID
        clrf    ACCbLO
        retlw   0
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Division ( 56 / 24 -> 24 )                                         -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCc, ACCd                                        -*
;*-                                                                        -*
;*-     56-bit dividend in ACCb,ACCc ( ACCb has msb's and ACCc has lsb's)  -*
;*-     24-bit divisor in ACCa                                             -*
;*-     quotient is stored in ACCc                                         -*
;*-     remainder is stored in ACCb                                        -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Div24
        call    Dsetup

dloop   bcf     STATUS,C
        rlf     ACCcLO,1         ;Rotate dividend left 1 bit position
        rlf     ACCcMID,1
        rlf     ACCcHI,1
        rlf     ACCbLO,1
        rlf     ACCbMID,1
        rlf     ACCbHI,1

        btfsc   STATUS,C        ;invert carry and exclusive or with the
        goto    clear           ;msb of the divisor then move this bit
        btfss   ACCaHI,msb_bit  ;into the lsb of the dividend
        incf    ACCcLO,1
        goto    cont
clear   btfsc   ACCaHI,msb_bit
        incf    ACCcLO,1

cont    btfsc   ACCcLO,lsb_bit  ;check the lsb of the dividend
        goto    minus
        call    Add24           ;if = 0, then add divisor to upper 24 bits
        goto    check           ;of dividend
minus   call    Sub24           ;if = 1, then subtract divisor from upper
                                ;24 bits of dividend

check   decfsz  temp,f          ;do 24 times
        goto    dloop

        bcf     STATUS,C
        rlf     ACCcLO,1          ;shift lower 24 bits of dividend 1 bit 
        rlf     ACCcMID,1         ;position left
        rlf     ACCcHI,1
        btfsc   ACCbHI,msb_bit  ;exlusive or the inverse of the msb of the
        goto    w1              ;dividend with the msb of the divisor
        btfss   ACCaHI,msb_bit  ;store in the lsb of the dividend
        incf    ACCcLO,1
        goto    wzd
w1      btfsc   ACCaHI,msb_bit
        incf    ACCcLO,1
wzd     btfss   ACCbHI,msb_bit  ;if the msb of the remainder is set and
        goto    wend
        btfsc   ACCaHI,msb_bit  ;the msb of the divisor is not
        goto    wend
        call    Add24           ;add the divisor to the remainder to correct
                                ;for zero partial remainder

wend    retlw   0               ;quotient in 24 lsb's of dividend
                                ;remainder in 24 msb's of dividend

Dsetup        
        movlw   0x18            ;loop 24 times
        movwf   temp
        
        retlw   0
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     2's Complement                                                     -*
;*-                                                                        -*
;*-     Uses ACCa                                                          -*
;*-                                                                        -*
;*-     Performs 2's complement conversion on ACCa                         -*
;*--------------------------------------------------------------------------*
;****************************************************************************
compA    
        comf    ACCaLO,1          ;invert all bits in accumulator a
        comf    ACCaMID,1
        comf    ACCaHI,1
        incf    ACCaLO,1          ;add one to accumulator a
        btfsc   STATUS,Z
        incf    ACCaMID,1
        btfsc   STATUS,Z
        incf    ACCaHI,1
        retlw   0
;___________________________________________________
;---------------------------CONVERSION FROM HEXADECIMAL TO DECIMAL---------------------------------------------------
;*		USES	ACCcLO AND ACCcMID
;*		RESULTS IN TENK (TENS OF THOUSANDS) THOUSANDS, HUNDREDS,TENS AND ONES
;*----------------------------------------------------------------------------------

DECIMAL											
		BANKSEL		PORTA			;change banks
		SWAPF		ACCcMID,W
		ANDLW		0FH
		ADDLW		0F0H
		MOVWF		THOUSANDS
		ADDWF		THOUSANDS,F
		ADDLW		0E2H
		MOVWF		HUNDREDS
		ADDLW		32H
		MOVWF		ONES

		MOVF		ACCcMID,0
		ANDLW		0FH
		ADDWF		HUNDREDS,F
		ADDWF		HUNDREDS,F
		ADDWF		ONES,F
		ADDLW		0E9H
		MOVWF		TENS
		ADDWF		TENS,F
		ADDWF		TENS,F

		SWAPF		ACCcLO,W
		ANDLW		0FH
		ADDWF		TENS,F
		ADDWF		ONES,F

		RLF			TENS,F
		RLF			ONES,F
		COMF		ONES,F
		RLF			ONES,F
			
		MOVF		ACCcLO,W
		ANDLW		0FH
		ADDWF		ONES,F
		RLF			THOUSANDS,F
		
		MOVLW		07H
		MOVWF		TENK

		MOVLW		0AH
LB1		ADDWF		ONES,F
		DECF		TENS,F
		BTFSS		3,0
		GOTO		LB1
LB2		ADDWF		TENS,F
		DECF		HUNDREDS,F
		BTFSS		3,0
		GOTO		LB2

LB3		ADDWF		HUNDREDS,F
		DECF		THOUSANDS,F
		BTFSS		3,0
		GOTO		LB3

LB4		ADDWF		THOUSANDS,F
		DECF		TENK,F
		BTFSS		3,0
		GOTO		LB4

		RETURN
;------------------------------------------------------------------------------

SEND	MOVWF		DATA1
		CALL		WRDATA_CHK
		CALL		DELAY30uS
		RETURN
WRDATA_CHK
		CALL		CHK_BUSY
		BSF			RS
		BCF			RW
		BSF			E
		MOVF		DATA1,0
		MOVWF		PORTB
		BCF			E
		MOVLW		.255
		MOVWF		PORTB
		RETURN
WRINS_CHK
		CALL		CHK_BUSY
WRINS_NOCHK
		BCF			RS
		BCF			RW
		BSF			E
		MOVF		DATA1,0
		MOVWF		PORTB
		BCF			E
		MOVLW		.255
		MOVWF		PORTB
		RETURN
CHK_BUSY
		BCF			RS			;RS LOW
		BSF			RW			;RW HIGH
		BSF			E			;ENABLE HIGH
		BANKSEL		TRISB		;CHANGING OUTPUT TO INPUT
		MOVLW		.255
		MOVWF		TRISB		;LOAD
		BANKSEL		PORTA		;CHANGE BANKS
		BANKSEL		TRISB		;CHANGING INPUT TO OUTPUT
		CLRF		TRISB		;LOAD
		BANKSEL		PORTA		;CHANGE BANKS
		BCF			E			;LOWER ENABLE
		RETURN					;RETURN

DELAY40MS
		MOVLW		.105
		MOVWF		COUNTER1
		DECFSZ		COUNTER0,1
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		RETURN
DELAY30uS
		MOVLW		.16
		MOVWF		COUNTER0
		DECFSZ		COUNTER0,1
		GOTO		$-1
		RETURN
DELAY2MS
		MOVLW		6
		MOVWF		COUNTER1
		DECFSZ		COUNTER0,1
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		RETURN

;***********************************INTIALIZING LCD DISPLAY*******************
DOG		CALL		DELAY40MS
		MOVLW		039H			;FUNCTION SET INSTRUCTION SET 0
		MOVWF		DATA1
		CALL		WRINS_NOCHK
		CALL		DELAY30uS
		MOVLW		01CH			;SET BIAS 5 VOLT
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		074H			;CONTRAST SET ADJUSTMENT 5 VOLT	
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		052H			;POWER/ICON/CONTRAST 5 VOLT
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		069H			;FOLLOWER CONTROL
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		038H			;FUNCTION SET  INSTRUCTION SET 1
		MOVWF		DATA1
		CALL		WRINS_NOCHK
		CALL		DELAY30uS
		MOVLW		00CH			;DISPLAY ON CURSOR OFF
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		001H			;CLEAR DISPLAY
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY2MS
		MOVLW		002H			;GO HOME
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY2MS
		MOVLW		006H			;ENTRY MODE SET
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS	
		RETURN

LOAD		
		MOVLW		053H		;Z
		CALL		SEND
		MOVLW		054H		;E
		CALL		SEND
		MOVLW		041H		;R
		CALL		SEND
		MOVLW		04EH		;O
		CALL		SEND
		MOVLW		044H		;I
		CALL		SEND
		MOVLW		00A0H		;N
		CALL		SEND
		MOVLW		042H		;G
		CALL		SEND


		RETURN

;**************************** DISPLAYING *****************************8

DISPLAYING	
		MOVLW	0				;LOAD INTO 0 THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS
		ADDWF	ONES,0			;ADD THE OFFSET TO THE LOW ADDRESS (ADRES)
		MOVLW	5				;LOAD 3 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC	STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW	1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF	PCLATH
    	MOVF 	ONES,W      	 ;MOVE CONTROL INTO W
    	CALL    DISPLAY          ;GO TO LOOK UP TABLE       
		MOVWF	DIGIT_1
		CLRF	PCLATH

		MOVLW		0
		ADDWF		TENS,0
		MOVLW		5
		BTFSC		STATUS,C
		ADDLW		1
		MOVWF		PCLATH
		MOVF		TENS,0
		CALL		DISPLAY
		MOVWF		DIGIT_2
		CLRF		PCLATH

		MOVLW		0
		ADDWF		HUNDREDS,0
		MOVLW		5
		BTFSC		STATUS,C
		ADDLW		1
		MOVWF		PCLATH
		MOVF		HUNDREDS,0
		CALL		DISPLAY
		MOVWF		DIGIT_3
		
		MOVLW		0
		ADDWF		THOUSANDS,0
		MOVLW		5
		BTFSC		STATUS,C
		ADDLW		1
		MOVWF		PCLATH
		MOVF		THOUSANDS,0
		CALL		DISPLAY
		MOVWF		DIGIT_4
		CLRF		PCLATH

		MOVLW		0
		ADDWF		TENK,0
		MOVLW		5
		BTFSC		STATUS,C
		ADDLW		1
		MOVWF		PCLATH
		MOVF		TENK,0
		CALL		DISPLAY
		MOVWF		DIGIT_5
		CLRF		PCLATH
		RETURN

		ORG	500
DISPLAY
		ADDWF	PCL,1				;ADD W TO PROGRAM COUNTER AND LOOK UP DATA  "." = DECIMAL #
		RETLW	B'00110000'			;0
		RETLW	B'00110001'			;1
		RETLW	B'00110010'			;2
		RETLW	B'00110011'			;3
		RETLW	B'00110100'			;4
		RETLW	B'00110101'			;5
		RETLW	B'00110110'			;6
		RETLW	B'00110111'			;7
		RETLW	B'00111000'			;8
		RETLW	B'00111001'			;9



DISPLAY_DELAY
		MOVLW		10
		MOVWF		TIMER3
		DECFSZ		TIMER1,1
		GOTO		$-1
		DECFSZ		TIMER2,1
		GOTO		$-3
		DECFSZ		TIMER3,1
		GOTO		$-5

		RETURN

;********************USED FOR TROUBLE SHOOTING******************
CHECK
		BANKSEL		PORTA
		MOVLW		043H		;C
		CALL		SEND
		MOVLW		048H		;H
		CALL		SEND
		MOVLW		045H		;E
		CALL		SEND
		MOVLW		043H		;C
		CALL		SEND
		MOVLW		04BH		;K
		CALL		SEND

				RETURN

RESET_REG
		BCF		INTCON,GIE
		CALL	DOG
		MOVLW	05AH				;Z
		CALL	SEND
		MOVLW	045H				;E
		CALL	SEND		
		MOVLW	052H				;R
		CALL	SEND
		MOVLW	04FH				;O
		CALL	SEND	 
 		MOVLW	049H				;I
		CALL	SEND
		MOVLW	04EH				;N
		CALL	SEND	
		MOVLW	047H				;G
		CALL	SEND
 		MOVLW	.60
		MOVWF	SECONDS
		MOVLW	.60
		MOVWF	MINUTES
		MOVLW	.24
		MOVWF	HOURS
		MOVLW	.1
		MOVWF	DAYS
		MOVLW	.244
		MOVWF	TIMER1
		CLRF	COUNT_LO
		CLRF	COUNT_MID
		CLRF	COUNT_DIS_LO
		CLRF	COUNT_DIS_MID
		CLRF	MIN_REM_HI
		CLRF	MIN_REM_MID
		CLRF	MIN_REM_LO
		CLRF	MIN_DIS_HI
		CLRF	MIN_DIS_MID
		CLRF	MIN_DIS_LO
		CLRF	HOUR_REM_HI
		CLRF	HOUR_REM_MID
		CLRF	HOUR_REM_LO
		CLRF	HOUR_DIS_MID
		CLRF	HOUR_DIS_HI
		CLRF	HOUR_DIS_LO
		CLRF	DAY_REM_HI
		CLRF	DAY_REM_MID
		CLRF	DAY_REM_LO
		CLRF	DAY_DIS_HI
		CLRF	DAY_DIS_MID
		CLRF	DAY_DIS_LO
		CLRF	ACCaHI  	           ;24-Bit accumulator a
		CLRF	ACCaMID 	
		CLRF	ACCaLO  	
		CLRF	ACCbHI  	           ;24-Bit accumulator b
		CLRF	ACCbMID 	
		CLRF	ACCbLO  	
		CLRF	ACCcHI  	           ;24-Bit accumulator c
		CLRF	ACCcMID 	
		CLRF	ACCcLO  	
		CLRF	ACCdHI  	           ;24-Bit accumulator d
		CLRF	ACCdMID 	
		CLRF	ACCdLO  	
		CLRF	TENK					;10,000
		CLRF	temp   	           		;temporary storage		
		
		BSF		INTCON,GIE
		RETURN
  END